@@ -15,20 +15,31 @@ if (get_local) {
     #         file = "./tests/testthat/bb_test.Rds")
 
     # Equivalent code using internal 'stub.R' function
-    base_url <- "https://nominatim.openstreetmap.org"
-    query <- list (q = 'Salzburg', viewbox = NULL, format = 'json',
-                   featuretype = 'settlement', key = NULL, limit = 10)
-    the_url <- httr::modify_url (base_url, query = query)
-    cfm_output_bb <- NULL
-    trace(
-          curl::curl_fetch_memory,
-          exit = function() {
-              cfm_output_bb <<- returnValue()
-          })
-    res <- httr::GET (the_url)
-    class (cfm_output_bb) <- 'response'
-    untrace (curl::curl_fetch_memory)
-    save (cfm_output_bb, file = '../cfm_output_bb.rda')
+    stub1 <- function (query)
+    {
+        base_url <- "https://nominatim.openstreetmap.org"
+        the_url <- httr::modify_url (base_url, query = query)
+        cfm_output <- NULL
+        trace(
+              curl::curl_fetch_memory,
+              exit = function() {
+                  cfm_output <<- returnValue()
+              })
+        res <- httr::GET (the_url)
+        class (cfm_output) <- 'response'
+        untrace (curl::curl_fetch_memory)
+        return (cfm_output)
+    }
+
+    list (q = 'Salzburg', viewbox = NULL, format = 'json',
+          featuretype = 'settlement', key = NULL, limit = 10) %>%
+        stub1 () -> cfm_output_bb
+    save (cfm_output_bb, file = '../cfm_output_bb1.rda')
+    list (q = 'Salzburg', viewbox = NULL, format = 'json',
+          polygon_text = 1,
+          featuretype = 'settlement', key = NULL, limit = 10) %>%
+        stub1 () -> cfm_output_bb
+    save (cfm_output_bb, file = '../cfm_output_bb2.rda')
 }
 
 context ("bbox")
@@ -45,7 +56,7 @@ test_that ('getbb-place_name', {
                if (has_internet) {
                    if (!test_all)
                    {
-                       load("../cfm_output_bb.rda")
+                       load("../cfm_output_bb1.rda")
                        stub (getbb, 'httr::GET', function (x) cfm_output_bb )
                    }
                    res <- getbb (place_name = "Salzburg")
@@ -56,6 +67,24 @@ test_that ('getbb-place_name', {
                }
           })
 
+test_that ('getbb-polygon', {
+               if (has_internet) {
+                   if (!test_all)
+                   {
+                       load("../cfm_output_bb2.rda")
+                       stub (getbb, 'httr::GET', function (x) cfm_output_bb )
+                   }
+                   res <- getbb (place_name = "Salzburg", format_out = "polygon")
+                   expect_is (res, "list")
+                   expect_true (all (lapply (res, nrow) > 2))
+                   expect_true (all (lapply (res, class) == "matrix"))
+                   res <- getbb (place_name = "Salzburg", format_out = "sf_polygon")
+                   expect_is (res, "sf")
+                   expect_is (res$geometry, "sfc_POLYGON")
+                   expect_length (res$geometry, 1)
+               }
+          })
+
 test_that ('bbox-to-string', {
                bb <- cbind (1:2, 3:4)
                expect_is (bbox_to_string (bb), "character")
